<?php
class FatalException extends Exception {};
class WarningException extends Exception {};

function exceptionErrorHandler($errno, $errstr, $errfile, $errline)  {
  switch ($errno) {
  case E_USER_ERROR:
   throw new FatalException($errstr, $errno);
   break;
  case E_USER_WARNING:
  case E_WARNING:
   throw new WarningException($errstr, $errno);
   break;
  case E_USER_NOTICE:
   //throw new WarningException($errstr, $errno);
   break;
  default:
     echo " - Unknown error - <b>$errstr</b>($errno) <br>";
   break;
  }
}

$old_error_handler = null;

function install_exception_errorhandler() {
   global $old_error_handler;
   $old_error_handler = set_error_handler('exceptionErrorHandler');
}

function uninstall_exception_errorhandler() {
   global $old_error_handler;
   set_error_handler($old_error_handler);
   $old_error_handler = null;
}
// returns formatted info-dump of an exception
function exception_dump(&$e) {
   $res = "";
   $res .= "<h3>{$e->getMessage()} ({$e->getCode()})</h3>\n\n";
   $res .= "file: {$e->getFile()}<br/>\n";
   $res .= "line: {$e->getLine()}<br/>\n";
   $res .= "<PRE>";
   $res .= $e->getTraceAsString();
   $res .= "</PRE><br>";
   return $res;
}

/*
 * uncomment to always dump sql statement in Exception message
 * (for debugging purposes only)
 * 
 */
 //define('SQL_DEBUG', 1);

// Base class for Database Exceptions
class DBException extends Exception {};

// Thrown when a Database connection fails
class DBConnectException extends DBException {}

// Thrown when problems with the execution or preparation of an SQL Query occurs.
class SQLException extends DBException {
     private $sql;
     private $sqlerrmsg;
    
     function __construct($msg, $code, $sql) {
           
            $this->sqlerrmsg = $msg;
            if (defined('SQL_DEBUG')) {
                $msg .= " in '$sql'";
            }
            /* Pass all arguments passed to
            * the constructor on to the parent's constructor 
            */
            parent::__construct($msg, $code);
            $this->sql = $sql;
       }
    
     function getSQL() {
            return $this->sql;
     }

     function getSQLError() {
            return $this->sqlerrmsg;
     }
    
}
?>