<?php
/*
 * retreat all data from POST, GET superglobal array
 * and validates its data type.
 * inputs: array of avaible variables names, data type
 */
class secureRequest
{
	/*
	 * set $reportWarnings true if you wanna get warnings
	 * of unknown request
	 * good for debugging purposes
	 */
	private $reportWarnings = false;
	private $secure = array();
	
    public function __construct($allowedVars) {
    	//check alowedVars if there is
    	//given $_GET, $_POST, COOKIE, FILES variable
		foreach ($allowedVars as $rowNum => $rowVal) {
			
			//default is not to automatically initialize variable
			if ($rowVal[4]!==true) $initialize = false;
			else $initialize = true;
			//reset the found state
			$found = false;
			
			switch ($rowVal[0]){
			// $_GET
    		case 'GET':
    			$superArray = $_GET;
    			$superArrayName = 'GET';
    		break;
			// $_POST
    		case 'POST':
    			$superArray = $_POST;
    			$superArrayName = 'POST';
    		break;
    		// $_COOKIE
    		case 'COOKIE':
    			$superArray = $_COOKIE;
    			$superArrayName = 'COOKIE';
    		break;
    		// $_FILES
    		case 'FILES':
    			$superArray = $_FILES;
    			$superArrayName = 'FILES';
    		break;
    		//other
    		default:
    			$superArray = array();
    			echo "<div><i>secureRequest Error</i> - 
    				 Unknown method: $rowVal[0]</div>\n";
			}	
			
			foreach($superArray AS $varName => $value) {
					//name is equal 
					if ($rowVal["1"]===$varName) {
						//datatype is equal
						$rowValType = $rowVal["2"];
						if ($this->validate($value,$rowValType)) {
							$this->secure[$varName] = $value;
							//we found the right request 
							$found = true;
						}
						//bad datatype - throw error
						else
							echo "<div><i>secureRequest Error</i> - 
							 Bad datatype in request: ".
				  			 $rowVal[0] ."['". $rowVal[1] ."']</div>\n";
					}
					//there is no variable in $allowedVars
					elseif ($this->$reportWarnings)
						echo "<div><i>secureRequest Warning</i> -". 
							 "Ignoring unknown request: ".
				  			 $rowVal[0] ."</div>\n";
			}
			
			//we want to initialize
			if ($initialize && !$found) {
				//initialize! - default value
				$name = $rowVal[1];
				$this->secure[$name] = $rowVal[3];
			}
			//error
			elseif (!$found) echo "<div><i>secureRequest Error</i> - 
					 No such variable in request: ".
					 $rowVal[0] ."['". $rowVal[1] ."']</div>\n";
		}
    }
    
    public function __get($name) {
        if (isset($this->secure[$name])) 
            return $this->secure[$name];
    }
	
    /*
     * Here is a possible hole
     * you could set another datatype
     * so look after what you set
     */
     public function __set($name, $val) {
        if (isset($this->secure[$name])) {
            $this->secure[$name] = $val;
        }
    }
	
    /*
     * ideal for testing if $secure['foo'] variable isset
     * example: if (isset($secureRequest->gId))
     * note $secureRequest is instance of class secureRequest
     */
    public function __isset($name){
        return isset($this->secure[$name]);
    }

    public function __unset($name){
		unset($this->secure[$name]);
    }
     
    /*
     * validates data from posts and get request
     * because everything from this is string
     * we have to cast to string or test with
     * is_numeric function the numerical expressions
     *
     */
    public function validate($var, $type) {
		switch ($type) {
			//integer
			case 'I':
				return 
				  (is_numeric($var) ? intval($var) == $var : false);

			break;
			//double
			case 'D':
				if ((string)(float)$var === (string)$var)
					return true;
				else
					return false;
			break;
			//boolean
			case 'B':
				if (is_bool($var) || $var=="true" || $var=="false")
					return true;
				else
					return false;
			break;
			//string
			case 'S':
				if (is_string($var))
					return true;
				else return false;
			break;
			//uploaded file
			case 'F':
				if (is_uploaded_file($var['tmp_name']))
					return true;
				else return false;
			break;
			//error			
			default:
				echo "Not valid data type<br />\n";
				return false;
			break;
		}
   }
  
}
     /*
      * $allowedVars - array of settings for class secureRequest 
   	  * 		(string $method,
   	  * 		 string $name,
   	  * 		 string $type,
   	  * 		 mixed $default,
   	  * 		 boolean $initiate)
      * 
      * $method   - possible values are: GET, POST, COOKIE, FILES
      * $name 	  -	desired name of a variable
      * 			good practise is give beginning letter with desired
      * 			method e.g. g,p,c,f
      * $type     - possible values are: 'I' as integer,
      * 			'D' as double/float,'B' as boolean, 'S' as string  
      * $default  - in case of $initiate=true, this is default
      * 			value for variable wich wasn't initiated
      * $initiate - if true variable will be initiated if it wasn't
      * 			from form
      */
     
$allowedVars = array(
					array ('GET','gId','I',0,true),
					array ('POST','pHeader','S','',false),
					array ('POST','pContent','S','',false),
					array ('FILES','fFile','F','',false),
					array ('POST','pSent','B','',false),
					array ('GET','gDouble','D',0,true)								   
				);

$secureVar = new secureRequest($allowedVars);

//not initialized with request but initilized with default value '0'
echo $secureVar->gId;
echo "\n<br /><br />\n";

//use setter carefully
$secureVar->gId =
	"setter doesn't go through validation process<br />";

//if we not sure we can still validate datatype
//result: not valid, we set gID to string 
if ($secureVar->validate($secureVar->gID,"I"))
	echo "valid<br />";
else
	echo "not valid<br />";
echo "\n<br />\n";

//if we not sure we can still validate
//result: valid, we set gID to string 
if ($secureVar->validate($secureVar->gId,"S"))
	echo "valid<br />";
else
	echo "not valid<br />";
echo $secureVar->gId;
echo "\n<br />\n";


//testing form sending
if (isset($secureVar->pSent)) {
	echo 'do some code here'."<br />\n";
}
echo "\n<br /><br />\n";

//simply getting the validated value
echo $secureVar->pHeader;

//getting not validated (we don't send this) value
//result we get no value 
echo $secureVar->pName;
?>